##
## Purpose:
## This class is used by any recipes that are built using
## Cargo.

require_script cargo_common_utils.sh

# the binary we will use
: ${CARGO:="cargo"}
: ${RUSTC:="rustc"}

# In case something fails in the build process, give a bit more feedback on
# where the issue occured
export RUST_BACKTRACE="1"

# The directory of the Cargo.toml relative to the root directory, per default
# assume there's a Cargo.toml directly in the root directory
: ${CARGO_SRC_DIR:=""}
# The actual path to the Cargo.toml
: ${MANIFEST_PATH:="${S}/${CARGO_SRC_DIR}/Cargo.toml"}
: ${RUSTFLAGS:=""}

BUILD_MODE="--release"
CARGO_BUILD_FLAGS="-v --target ${HOST_SYS} ${BUILD_MODE} --manifest-path=${MANIFEST_PATH}"

# This is based on the content of CARGO_BUILD_FLAGS and generally will need to
# change if CARGO_BUILD_FLAGS changes.
BUILD_DIR="release"
CARGO_TARGET_SUBDIR="${HOST_SYS}/${BUILD_DIR}"

cargo_build () {
	export RUSTFLAGS="${RUSTFLAGS}"
	export RUST_TARGET_PATH="${RUST_TARGET_PATH}"
	log "cargo = $(which ${CARGO})"
	log "rustc = $(which ${RUSTC})"
	log "${CARGO} build ${CARGO_BUILD_FLAGS} $@"
	"${CARGO}" build ${CARGO_BUILD_FLAGS} "$@"
}

cargo_do_configure () {
	cargo_common_do_configure
}

cargo_do_compile () {
	cargo_fix_env
	cargo_build
}

cargo_do_install () {
	local have_installed=false
	for tgt in "${B}/target/${CARGO_TARGET_SUBDIR}/"*; do
		case $tgt in
		*.so|*.rlib)
			log_exec install -d "${D}${rustlibdir}"
			log_exec install -m755 "$tgt" "${D}${rustlibdir}"
			have_installed=true
			;;
		*examples)
			if [ -d "$tgt" ]; then
				for example in "$tgt/"*; do
					if [ -f "$example" ] && [ -x "$example" ]; then
						log_exec install -d "${D}${bindir}"
						log_exec install -m755 "$example" "${D}${bindir}"
						have_installed=true
					fi
				done
			fi
			;;
		*)
			if [ -f "$tgt" ] && [ -x "$tgt" ]; then
				log_exec install -d "${D}${bindir}"
				log_exec install -m755 "$tgt" "${D}${bindir}"
				have_installed=true
			fi
			;;
		esac
	done
	if ! $have_installed; then
		fatal 1 "Did not find anything to install"
	fi
}

cargo_do_fetch () {
	mkdir -p ${CARGO_HOME}/bitbake
	local cratesDir=$DOWNLOAD_LOC/crates
	mkdir -p ${cratesDir}

	for uri in $CRATE_SRC_URIS; do
		local protocol=$(echo "$uri" | grep "://" | sed -e's,^\(.*://\).*,\1,g')
		# Remove the protocol
		local url_no_protocol=$(echo "${uri/$protocol/}")
		# Use tr: Make the protocol lower-case for easy string compare
		protocol=$(echo "$protocol" | tr '[:upper:]' '[:lower:]')

		# Iterate to next URI if the protocol is not supported
		if [ "$protocol" != "crate://" ]; then continue; fi

		local version=${url_no_protocol##*/}
		local name=${url_no_protocol%/$version}
		name=${name##*/}
		local host=${url_no_protocol%/$name/$version}

		if [ "$host" == "crates.io" ]; then
			host="crates.io/api/v1/crates"
		fi
		
		local srcUrl="https://${host}/${name}/${version}/download"
		local downloadFile="${name}-${version}.crate"

		if [ -f "${cratesDir}/${downloadFile}" ]; then
			log "$downloadFile already found in ${cratesDir}"
		else
			log "Downloading $downloadFile..."
			log_exec wget --no-check-certificate -nv --output-document="${cratesDir}/$downloadFile" "$srcUrl"
		fi
	done
}

cargo_do_unpack () {
	mkdir -p ${CARGO_HOME}/bitbake
	local cratesDir=$DOWNLOAD_LOC/crates
	mkdir -p ${cratesDir}

	for uri in $CRATE_SRC_URIS; do
		local protocol=$(echo "$uri" | grep "://" | sed -e's,^\(.*://\).*,\1,g')
		# Remove the protocol
		local url_no_protocol=$(echo "${uri/$protocol/}")
		# Use tr: Make the protocol lower-case for easy string compare
		protocol=$(echo "$protocol" | tr '[:upper:]' '[:lower:]')

		# Iterate to next URI if the protocol is not supported
		if [ "$protocol" != "crate://" ]; then continue; fi

		local version=${url_no_protocol##*/}
		local name=${url_no_protocol%/$version}
		name=${name##*/}

		local crateFile="${name}-${version}.crate"
		local crateFileHash=($(sha256sum ${cratesDir}/${crateFile}))

		log "Unpack $crateFile..."
		log_exec tar -xz --no-same-owner -f "${cratesDir}/${crateFile}" -C "${CARGO_HOME}/bitbake"

		cat <<- EOF > "${CARGO_HOME}/bitbake/${name}-${version}/.cargo-checksum.json"
		{"files": {}, "package": "$crateFileHash"}
		EOF
	done
}
